@@ -192,6 +192,14 @@ struct loader_device {
 
     VkAllocationCallbacks alloc_callbacks;
 
+    // List of activated device extensions that have terminators implemented in the loader
+    struct {
+        bool khr_swapchain_enabled;
+        bool khr_display_swapchain_enabled;
+        bool ext_debug_marker_enabled;
+        bool ext_debug_utils_enabled;
+    } extensions;
+
     struct loader_device *next;
 };
 
@@ -304,6 +312,12 @@ struct loader_instance {
 #endif
 #ifdef VK_USE_PLATFORM_ANDROID_KHR
     bool wsi_android_surface_enabled;
+#endif
+#ifdef VK_USE_PLATFORM_MACOS_MVK
+    bool wsi_macos_surface_enabled;
+#endif
+#ifdef VK_USE_PLATFORM_IOS_MVK
+    bool wsi_ios_surface_enabled;
 #endif
     bool wsi_display_enabled;
 };
